@@ -6,6 +6,7 @@ import os
 import subprocess
 import sys
 import tempfile
+import textwrap
 import unittest
 from test import support
 from test.support.script_helper import (
@@ -257,6 +258,21 @@ class CmdLineTest(unittest.TestCase):
         self.assertEqual(stdout, expected)
         self.assertEqual(p.returncode, 0)
 
+    def test_non_interactive_output_buffering(self):
+        code = textwrap.dedent("""
+            import sys
+            out = sys.stdout
+            print(out.isatty(), out.write_through, out.line_buffering)
+            err = sys.stderr
+            print(err.isatty(), err.write_through, err.line_buffering)
+        """)
+        args = [sys.executable, '-c', code]
+        proc = subprocess.run(args, stdout=subprocess.PIPE,
+                              stderr=subprocess.PIPE, text=True, check=True)
+        self.assertEqual(proc.stdout,
+                         'False False False\n'
+                         'False False True\n')
+
     def test_unbuffered_output(self):
         # Test expected operation of the '-u' switch
         for stream in ('stdout', 'stderr'):
@@ -370,10 +386,10 @@ class CmdLineTest(unittest.TestCase):
 
         if sys.platform == 'win32':
             self.assertEqual(b'1\r\n2\r\n', out)
-            self.assertEqual(b'3\r\n4', err)
+            self.assertEqual(b'3\r\n4\r\n', err)
         else:
             self.assertEqual(b'1\n2\n', out)
-            self.assertEqual(b'3\n4', err)
+            self.assertEqual(b'3\n4\n', err)
 
     def test_unmached_quote(self):
         # Issue #10206: python program starting with unmatched quote
@@ -429,7 +445,7 @@ class CmdLineTest(unittest.TestCase):
             stderr=subprocess.PIPE,
             preexec_fn=preexec)
         out, err = p.communicate()
-        self.assertEqual(support.strip_python_stderr(err), b'')
+        self.assertEqual(err, b'')
         self.assertEqual(p.returncode, 42)
 
     def test_no_stdin(self):
@@ -788,41 +804,6 @@ class CmdLineTest(unittest.TestCase):
         self.assertTrue(proc.stderr.startswith(err_msg), proc.stderr)
         self.assertNotEqual(proc.returncode, 0)
 
-    def test_int_max_str_digits(self):
-        code = "import sys; print(sys.flags.int_max_str_digits, sys.get_int_max_str_digits())"
-
-        assert_python_failure('-X', 'int_max_str_digits', '-c', code)
-        assert_python_failure('-X', 'int_max_str_digits=foo', '-c', code)
-        assert_python_failure('-X', 'int_max_str_digits=100', '-c', code)
-        assert_python_failure('-X', 'int_max_str_digits', '-c', code,
-                              PYTHONINTMAXSTRDIGITS='4000')
-
-        assert_python_failure('-c', code, PYTHONINTMAXSTRDIGITS='foo')
-        assert_python_failure('-c', code, PYTHONINTMAXSTRDIGITS='100')
-
-        def res2int(res):
-            out = res.out.strip().decode("utf-8")
-            return tuple(int(i) for i in out.split())
-
-        res = assert_python_ok('-c', code)
-        self.assertEqual(res2int(res), (-1, sys.get_int_max_str_digits()))
-        res = assert_python_ok('-X', 'int_max_str_digits=0', '-c', code)
-        self.assertEqual(res2int(res), (0, 0))
-        res = assert_python_ok('-X', 'int_max_str_digits=4000', '-c', code)
-        self.assertEqual(res2int(res), (4000, 4000))
-        res = assert_python_ok('-X', 'int_max_str_digits=100000', '-c', code)
-        self.assertEqual(res2int(res), (100000, 100000))
-
-        res = assert_python_ok('-c', code, PYTHONINTMAXSTRDIGITS='0')
-        self.assertEqual(res2int(res), (0, 0))
-        res = assert_python_ok('-c', code, PYTHONINTMAXSTRDIGITS='4000')
-        self.assertEqual(res2int(res), (4000, 4000))
-        res = assert_python_ok(
-            '-X', 'int_max_str_digits=6000', '-c', code,
-            PYTHONINTMAXSTRDIGITS='4000'
-        )
-        self.assertEqual(res2int(res), (6000, 6000))
-
 
 @unittest.skipIf(interpreter_requires_environment(),
                  'Cannot run -I tests when PYTHON env vars are required.')
@@ -860,9 +841,9 @@ class IgnoreEnvironmentTest(unittest.TestCase):
         )
 
 
-def test_main():
-    support.run_unittest(CmdLineTest, IgnoreEnvironmentTest)
+def tearDownModule():
     support.reap_children()
 
+
 if __name__ == "__main__":
-    test_main()
+    unittest.main()
