@@ -189,19 +189,6 @@ if 1:
         self.assertEqual(eval("0o777"), 511)
         self.assertEqual(eval("-0o0000010"), -8)
 
-    def test_int_literals_too_long(self):
-        n = 3000
-        source = f"a = 1\nb = 2\nc = {'3'*n}\nd = 4"
-        with support.adjust_int_max_str_digits(n):
-            compile(source, "<long_int_pass>", "exec")  # no errors.
-        with support.adjust_int_max_str_digits(n-1):
-            with self.assertRaises(SyntaxError) as err_ctx:
-                compile(source, "<long_int_fail>", "exec")
-            exc = err_ctx.exception
-            self.assertEqual(exc.lineno, 3)
-            self.assertIn('Exceeds the limit ', str(exc))
-            self.assertIn(' Consider hexadecimal ', str(exc))
-
     def test_unary_minus(self):
         # Verify treatment of unary minus on negative numbers SF bug #660455
         if sys.maxsize == 2147483647:
@@ -763,6 +750,16 @@ if 1:
             self.assertEqual(None, opcodes[0].argval)
             self.assertEqual('RETURN_VALUE', opcodes[1].opname)
 
+    def test_big_dict_literal(self):
+        # The compiler has a flushing point in "compiler_dict" that calls compiles
+        # a portion of the dictionary literal when the loop that iterates over the items
+        # reaches 0xFFFF elements but the code was not including the boundary element,
+        # dropping the key at position 0xFFFF. See bpo-41531 for more information
+
+        dict_size = 0xFFFF + 1
+        the_dict = "{" + ",".join(f"{x}:{x}" for x in range(dict_size)) + "}"
+        self.assertEqual(len(eval(the_dict)), dict_size)
+
 class TestExpressionStackSize(unittest.TestCase):
     # These tests check that the computed stack size for a code object
     # stays within reasonable bounds (see issue #21523 for an example
