@@ -7,19 +7,17 @@ import sys
 try:
     import os
 except:
-    print """Could not import the standard "os" module.
-  Please check your PYTHONPATH environment variable."""
+    print("""Could not import the standard "os" module.
+  Please check your PYTHONPATH environment variable.""")
     sys.exit(1)
 
 try:
     import symbol
 except:
-    print """Could not import the standard "symbol" module.  If this is
-  a PC, you should add the dos_8x3 directory to your PYTHONPATH."""
+    print("""Could not import the standard "symbol" module.  If this is
+  a PC, you should add the dos_8x3 directory to your PYTHONPATH.""")
     sys.exit(1)
 
-import os
-
 for dir in sys.path:
     file = os.path.join(dir, "os.py")
     if os.path.isfile(file):
@@ -28,5 +26,5 @@ for dir in sys.path:
             # Add the "test" directory to PYTHONPATH.
             sys.path = sys.path + [test]
 
-import regrtest # Standard Python tester.
-regrtest.main()
+import libregrtest # Standard Python tester.
+libregrtest.main()
